from zope.formlib.form import Action
from Products.Five import zcml
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from plone.app.workflow.browser.sharing import SharingView
from plone.app.controlpanel import form, maintenance, ram
from plone.protect import CheckAuthenticator
from zExceptions import Forbidden


# generate wrapper to check csrf-protection token for a given view method
def patchAction(action):
    assert isinstance(action, Action)   # make sure it's a formlib action
    original = action.success_handler
    def protect(self, *args, **kw):
        CheckAuthenticator(self.request)
        return original(self, *args, **kw)
    action.success_handler = protect


def patchViews():
    """ patch browser views which need to contain the authenticator token """

    # folder contents view
    import Products.PloneHotfixCVE20080164.browser
    zcml.load_config('configure.zcml', Products.PloneHotfixCVE20080164.browser)

    # sharing view
    original_call = SharingView.__call__
    def __call__(self):
        get = self.request.form.get
        submitted = get('form.submitted', False)
        cancel_button = get('form.button.Cancel', None) is not None
        if submitted and not cancel_button:
            authenticator = self.context.restrictedTraverse('@@authenticator', None) 
            if not authenticator.verify(): 
                raise Forbidden
        return original_call(self)
    __call__.__doc__ = original_call.__doc__
    SharingView.__call__ = __call__
    SharingView.template = ViewPageTemplateFile('sharing.pt')

    # common control panel configlet
    tmpl = ViewPageTemplateFile('control-panel.pt')
    form._template = tmpl
    form.controlpanel_named_template_adapter.default_template = tmpl
    patchAction(form.ControlPanelForm.handle_edit_action)

    # maintenance control panel configlet
    mcp = maintenance.MaintenanceControlPanel
    mcp.template = ZopeTwoPageTemplateFile('maintenance.pt')
    patchAction(mcp.handle_edit_action)
    patchAction(mcp.handle_shutdown_action)
    patchAction(mcp.handle_restart_action)

    # ram cache control panel configlet
    rcp = ram.RAMCacheControlPanel
    rcp.template = ZopeTwoPageTemplateFile('ram.pt')
    patchAction(rcp.handle_clearall_action)
    patchAction(rcp.handle_edit_action)

